AdderType : (numargs : Nat) -> Type
AdderType Z = Int
AdderType (S k) = (next : Int) -> AdderType k

adder : (numargs : Nat) -> (acc : Int) -> AdderType numargs
adder 0 acc = acc
adder (S k) acc = \next => adder k (next + acc)

data Format = Number Format
            | Float Format
            | Str Format
            | Chr Format
            | Lit String Format
            | End

PrintfType : Format -> Type
PrintfType (Number next) = (i : Int) -> PrintfType next
PrintfType (Float next) = (f : Double) -> PrintfType next
PrintfType (Str next) = (s : String) -> PrintfType next
PrintfType (Chr next) = (c : Char) -> PrintfType next
PrintfType (Lit _ next) = PrintfType next
PrintfType End = String

printfFmt : (fmt : Format) -> (acc : String) -> PrintfType fmt
printfFmt (Number next) acc = \i => printfFmt next (acc ++ show i)
printfFmt (Float next) acc = \f => printfFmt next (acc ++ cast f)
printfFmt (Str next) acc = \s => printfFmt next (acc ++ s)
printfFmt (Chr next) acc = \c => printfFmt next (acc ++ cast c)
printfFmt (Lit lit next) acc = printfFmt next (acc ++ lit)
printfFmt End acc = acc

toFormat : (xs : List Char) -> Format
toFormat [] = End
toFormat ('%' :: 'd' :: chars) = Number (toFormat chars)
toFormat ('%' :: 'f' :: chars) = Float (toFormat chars)
toFormat ('%' :: 's' :: chars) = Str (toFormat chars)
toFormat ('%' :: 'c' :: chars) = Chr (toFormat chars)
toFormat ('%' :: chars) = Lit "%" (toFormat chars)
toFormat (c :: chars) = case toFormat chars of
                             Lit lit chars' => Lit (strCons c lit) chars'
                             fmt => Lit (strCons c "") fmt

printf : (fmt : String) -> PrintfType (toFormat (unpack fmt))
printf fmt = printfFmt _ ""
